@@ -1,19 +1,19 @@
+use std::convert::TryInto;
 use std::ffi::{CStr, CString};
-use std::fs::File;
 use std::os::raw::c_void;
 #[cfg(unix)]
 use std::os::unix::io::AsRawFd;
 #[cfg(windows)]
 use std::os::windows::io::AsRawHandle;
-use yara_sys::{YR_SCANNER, YR_SCAN_CONTEXT};
+
+use yara_sys::{YR_SCAN_CONTEXT, YR_SCANNER};
 
 use crate::errors::*;
 use crate::Rule;
-use std::convert::TryInto;
 
-#[derive(Copy, Clone, Debug, Eq, PartialEq)]
-enum CallbackMsg {
-    RuleMatching,
+#[derive(Debug)]
+pub enum CallbackMsg<'r> {
+    RuleMatching(Rule<'r>),
     RuleNotMatching,
     ScanFinished,
     ImportModule,
@@ -21,13 +21,16 @@ enum CallbackMsg {
     UnknownMsg,
 }
 
-impl CallbackMsg {
-    pub fn from_yara(code: i32) -> Self {
+impl<'r> CallbackMsg<'r> {
+    fn from_yara(context: *mut YR_SCAN_CONTEXT, message: i32, message_data: *mut c_void) -> Self {
         use self::CallbackMsg::*;
-        let code = code as u32;
 
-        match code {
-            yara_sys::CALLBACK_MSG_RULE_MATCHING => RuleMatching,
+        match message as u32 {
+            yara_sys::CALLBACK_MSG_RULE_MATCHING => {
+                let rule = unsafe { &*(message_data as *mut yara_sys::YR_RULE) };
+                let context = unsafe { &*context };
+                RuleMatching(Rule::from((context, rule)))
+            }
             yara_sys::CALLBACK_MSG_RULE_NOT_MATCHING => RuleNotMatching,
             yara_sys::CALLBACK_MSG_SCAN_FINISHED => ScanFinished,
             yara_sys::CALLBACK_MSG_IMPORT_MODULE => ImportModule,
@@ -38,11 +41,9 @@ impl CallbackMsg {
 }
 
 #[derive(Copy, Clone, Debug, Eq, PartialEq)]
-enum CallbackReturn {
+pub enum CallbackReturn {
     Continue,
-    #[allow(dead_code)]
     Abort,
-    #[allow(dead_code)]
     Error,
 }
 
@@ -64,8 +65,9 @@ pub fn rules_scan_mem<'a>(
     mem: &[u8],
     timeout: i32,
     flags: i32,
-) -> Result<Vec<Rule<'a>>, YaraError> {
-    let mut results = Vec::<Rule<'a>>::new();
+    callback: impl FnMut(CallbackMsg<'a>) -> CallbackReturn,
+) -> Result<(), YaraError> {
+    let p_callback: Box<dyn FnMut(CallbackMsg<'a>) -> CallbackReturn> = Box::new(callback);
     let result = unsafe {
         yara_sys::yr_rules_scan_mem(
             rules,
@@ -73,14 +75,14 @@ pub fn rules_scan_mem<'a>(
             mem.len().try_into().unwrap(),
             flags,
             Some(scan_callback),
-            &mut results as *mut Vec<_> as *mut c_void,
+            &p_callback as *const Box<dyn FnMut(CallbackMsg<'a>) -> CallbackReturn> as *mut _,
             timeout,
         )
     };
 
     yara_sys::Error::from_code(result)
         .map_err(|e| e.into())
-        .map(|_| results)
+        .map(|_| ())
 }
 
 /// Scan a buffer with the provided YR_SCANNER and its defined external vars.
@@ -90,92 +92,72 @@ pub fn rules_scan_mem<'a>(
 pub fn scanner_scan_mem<'a>(
     scanner: *mut yara_sys::YR_SCANNER,
     mem: &[u8],
-) -> Result<Vec<Rule<'a>>, YaraError> {
-    let mut results = Vec::<Rule<'a>>::new();
+    callback: impl FnMut(CallbackMsg<'a>) -> CallbackReturn,
+) -> Result<(), YaraError> {
+    let p_callback: Box<dyn FnMut(CallbackMsg<'a>) -> CallbackReturn> = Box::new(callback);
     let result = unsafe {
         yara_sys::yr_scanner_set_callback(
             scanner,
             Some(scan_callback),
-            &mut results as *mut Vec<_> as *mut c_void,
+            &p_callback as *const Box<dyn FnMut(CallbackMsg<'a>) -> CallbackReturn> as *mut _,
         );
         yara_sys::yr_scanner_scan_mem(scanner, mem.as_ptr(), mem.len().try_into().unwrap())
     };
-
     yara_sys::Error::from_code(result)
         .map_err(|e| e.into())
-        .map(|_| results)
-}
-
-pub fn rules_scan_file<'a>(
-    rules: *mut yara_sys::YR_RULES,
-    file: &File,
-    timeout: i32,
-    flags: i32,
-) -> Result<Vec<Rule<'a>>, YaraError> {
-    let mut results = Vec::<Rule<'a>>::new();
-    let result = rules_scan_raw(rules, file, timeout, flags, &mut results);
-
-    yara_sys::Error::from_code(result)
-        .map_err(|e| e.into())
-        .map(|_| results)
-}
-
-/// Scan a file with the provided YR_SCANNER and its defined external vars.
-///
-/// Setting the callback function modifies the Scanner with no locks preventing
-/// data races, so it should only be called from a &mut Scanner.
-pub fn scanner_scan_file<'a>(
-    scanner: *mut yara_sys::YR_SCANNER,
-    file: &File,
-) -> Result<Vec<Rule<'a>>, YaraError> {
-    let mut results = Vec::<Rule<'a>>::new();
-    let result = scanner_scan_raw(scanner, file, &mut results);
-
-    yara_sys::Error::from_code(result)
-        .map_err(|e| e.into())
-        .map(|_| results)
+        .map(|_| ())
 }
 
 #[cfg(unix)]
-pub fn rules_scan_raw(
+pub fn rules_scan_file<'a, F: AsRawFd>(
     rules: *mut yara_sys::YR_RULES,
-    file: &File,
+    file: &F,
     timeout: i32,
     flags: i32,
-    results: &mut Vec<Rule>,
-) -> i32 {
+    callback: impl FnMut(CallbackMsg<'a>) -> CallbackReturn,
+) -> Result<(), YaraError> {
     let fd = file.as_raw_fd();
-    unsafe {
+    let p_callback: Box<dyn FnMut(CallbackMsg<'a>) -> CallbackReturn> = Box::new(callback);
+
+    let result = unsafe {
         yara_sys::yr_rules_scan_fd(
             rules,
             fd,
             flags,
             Some(scan_callback),
-            results as *mut Vec<_> as *mut c_void,
+            &p_callback as *const Box<dyn FnMut(CallbackMsg<'a>) -> CallbackReturn> as *mut _,
             timeout,
         )
-    }
+    };
+    yara_sys::Error::from_code(result)
+        .map_err(|e| e.into())
+        .map(|_| ())
 }
 
 #[cfg(windows)]
-pub fn rules_scan_raw(
+pub fn rules_scan_file<'a, F: AsRawHandle>(
     rules: *mut yara_sys::YR_RULES,
-    file: &File,
+    file: &F,
     timeout: i32,
     flags: i32,
-    results: &mut Vec<Rule>,
-) -> i32 {
+    callback: impl FnMut(CallbackMsg<'a>) -> CallbackReturn,
+) -> Result<(), YaraError> {
     let handle = file.as_raw_handle();
-    unsafe {
+    let p_callback: Box<dyn FnMut(CallbackMsg<'a>) -> CallbackReturn> = Box::new(callback);
+
+    let result = unsafe {
         yara_sys::yr_rules_scan_fd(
             rules,
             handle,
             flags,
             Some(scan_callback),
-            results as *mut Vec<_> as *mut c_void,
+            &p_callback as *const Box<dyn FnMut(CallbackMsg<'a>) -> CallbackReturn> as *mut _,
             timeout,
         )
-    }
+    };
+    yara_sys::Error::from_code(result)
+        .map_err(|e| e.into())
+        .map(|_| ())
 }
 
 #[cfg(unix)]
@@ -183,20 +165,25 @@ pub fn rules_scan_raw(
 ///
 /// Setting the callback function modifies the Scanner with no locks preventing
 /// data races, so it should only be called from a &mut Scanner.
-pub fn scanner_scan_raw<'a>(
+pub fn scanner_scan_file<'a, F: AsRawFd>(
     scanner: *mut yara_sys::YR_SCANNER,
-    file: &File,
-    results: &mut Vec<Rule>,
-) -> i32 {
+    file: &F,
+    callback: impl FnMut(CallbackMsg<'a>) -> CallbackReturn,
+) -> Result<(), YaraError> {
     let fd = file.as_raw_fd();
-    unsafe {
+    let p_callback: Box<dyn FnMut(CallbackMsg<'a>) -> CallbackReturn> = Box::new(callback);
+
+    let result = unsafe {
         yara_sys::yr_scanner_set_callback(
             scanner,
             Some(scan_callback),
-            results as *mut Vec<_> as *mut c_void,
+            &p_callback as *const Box<dyn FnMut(CallbackMsg<'a>) -> CallbackReturn> as *mut _,
         );
         yara_sys::yr_scanner_scan_fd(scanner, fd)
-    }
+    };
+    yara_sys::Error::from_code(result)
+        .map_err(|e| e.into())
+        .map(|_| ())
 }
 
 #[cfg(windows)]
@@ -204,20 +191,25 @@ pub fn scanner_scan_raw<'a>(
 ///
 /// Setting the callback function modifies the Scanner with no locks preventing
 /// data races, so it should only be called from a &mut Scanner.
-pub fn scanner_scan_raw<'a>(
+pub fn scanner_scan_file<'a, F: AsRawHandle>(
     scanner: *mut yara_sys::YR_SCANNER,
-    file: &File,
-    results: &mut Vec<Rule>,
-) -> i32 {
+    file: &F,
+    callback: impl FnMut(CallbackMsg<'a>) -> CallbackReturn,
+) -> Result<(), YaraError> {
     let handle = file.as_raw_handle();
-    unsafe {
+    let p_callback: Box<dyn FnMut(CallbackMsg<'a>) -> CallbackReturn> = Box::new(callback);
+
+    let result = unsafe {
         yara_sys::yr_scanner_set_callback(
             scanner,
             Some(scan_callback),
-            results as *mut Vec<_> as *mut c_void,
+            &p_callback as *const Box<dyn FnMut(CallbackMsg<'a>) -> CallbackReturn> as *mut _,
         );
         yara_sys::yr_scanner_scan_fd(scanner, handle)
-    }
+    };
+    yara_sys::Error::from_code(result)
+        .map_err(|e| e.into())
+        .map(|_| ())
 }
 
 /// Attach a process, pause it, and scan its memory.
@@ -226,22 +218,23 @@ pub fn rules_scan_proc<'a>(
     pid: u32,
     timeout: i32,
     flags: i32,
-) -> Result<Vec<Rule<'a>>, YaraError> {
-    let mut results = Vec::<Rule<'a>>::new();
+    callback: impl FnMut(CallbackMsg<'a>) -> CallbackReturn,
+) -> Result<(), YaraError> {
+    let p_callback: Box<dyn FnMut(CallbackMsg<'a>) -> CallbackReturn> = Box::new(callback);
     let result = unsafe {
         yara_sys::yr_rules_scan_proc(
             rules,
             pid as i32,
             flags,
             Some(scan_callback),
-            &mut results as *mut Vec<_> as *mut c_void,
+            &p_callback as *const Box<dyn FnMut(CallbackMsg<'a>) -> CallbackReturn> as *mut _,
             timeout,
         )
     };
 
     yara_sys::Error::from_code(result)
         .map_err(|e| e.into())
-        .map(|_| results)
+        .map(|_| ())
 }
 
 /// Attach a process, pause it, and scan its memory with the provided YR_SCANNER
@@ -252,20 +245,21 @@ pub fn rules_scan_proc<'a>(
 pub fn scanner_scan_proc<'a>(
     scanner: *mut yara_sys::YR_SCANNER,
     pid: u32,
-) -> Result<Vec<Rule<'a>>, YaraError> {
-    let mut results = Vec::<Rule<'a>>::new();
+    callback: impl FnMut(CallbackMsg<'a>) -> CallbackReturn,
+) -> Result<(), YaraError> {
+    let p_callback: Box<Box<dyn FnMut(CallbackMsg<'a>) -> CallbackReturn>> =
+        Box::new(Box::new(callback));
+    let user_data = Box::into_raw(p_callback) as *mut c_void;
     let result = unsafe {
-        yara_sys::yr_scanner_set_callback(
-            scanner,
-            Some(scan_callback),
-            &mut results as *mut Vec<_> as *mut c_void,
-        );
+        yara_sys::yr_scanner_set_callback(scanner, Some(scan_callback), user_data);
         yara_sys::yr_scanner_scan_proc(scanner, pid as i32)
     };
+    let _: Box<Box<dyn FnMut(CallbackMsg<'a>) -> CallbackReturn>> =
+        unsafe { Box::from_raw(user_data as *mut _) };
 
     yara_sys::Error::from_code(result)
         .map_err(|e| e.into())
-        .map(|_| results)
+        .map(|_| ())
 }
 
 extern "C" fn scan_callback(
@@ -274,16 +268,10 @@ extern "C" fn scan_callback(
     message_data: *mut c_void,
     user_data: *mut c_void,
 ) -> i32 {
-    let message = CallbackMsg::from_yara(message);
-    let rules = unsafe { &mut *(user_data as *mut Vec<Rule>) };
-
-    if message == CallbackMsg::RuleMatching {
-        let rule = unsafe { &*(message_data as *mut yara_sys::YR_RULE) };
-        let context = unsafe { &*context };
-        rules.push(Rule::from((context, rule)));
-    }
-
-    CallbackReturn::Continue.to_yara()
+    let message = CallbackMsg::from_yara(context, message, message_data);
+    let callback: &mut Box<dyn FnMut(CallbackMsg) -> CallbackReturn> =
+        unsafe { std::mem::transmute(user_data) };
+    callback(message).to_yara()
 }
 
 /// Setting the flags modifies the Scanner with no locks preventing data races,
