@@ -45,6 +45,17 @@
 //! [Yara-doc]: https://yara.readthedocs.io/en/stable/gettingstarted.html
 //! [polydet]: https://github.com/Polydet/polydet/
 
+pub use internals::ConfigName;
+use internals::{get_configuration, set_configuration};
+
+pub use crate::compiler::Compiler;
+pub use crate::errors::*;
+use crate::initialize::InitializationToken;
+pub use crate::matches::Match;
+pub use crate::rules::*;
+pub use crate::scanner::Scanner;
+pub use crate::string::YrString;
+
 mod compiler;
 mod initialize;
 mod internals;
@@ -55,15 +66,6 @@ mod string;
 
 pub mod errors;
 
-use crate::initialize::InitializationToken;
-
-pub use crate::compiler::Compiler;
-pub use crate::errors::*;
-pub use crate::matches::Match;
-pub use crate::rules::*;
-pub use crate::scanner::Scanner;
-pub use crate::string::YrString;
-
 /// Yara initialization token.
 ///
 /// Act as an initialization token to keep the library initialized.
@@ -134,6 +136,16 @@ impl Yara {
         InitializationToken::new().map(|token| Yara { _token: token })
     }
 
+    /// Set the configuration variable
+    pub fn set_configuration(&self, name: ConfigName, value: u32) -> Result<(), YaraError> {
+        set_configuration(name, value)
+    }
+
+    /// Get the configuration variable
+    pub fn get_configuration(&self, name: ConfigName) -> Result<u32, YaraError> {
+        get_configuration(name)
+    }
+
     /// Create and initialize the library.
     #[deprecated = "Use new"]
     pub fn create() -> Result<Yara, YaraError> {
